#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.
""" Account utilities """

import logging

from skale.transactions.tools import send_eth
from skale.utils.constants import LONG_LINE
from skale.wallets import LedgerWallet, Web3Wallet
from skale.utils.web3_utils import check_receipt, wait_for_receipt_by_blocks

logger = logging.getLogger(__name__)


WALLET_TYPE_TO_CLASS = {
    'ledger': LedgerWallet,
    'web3': Web3Wallet
}


def create_wallet(wallet_type='web3', *args, **kwargs):
    return WALLET_TYPE_TO_CLASS[wallet_type](*args, **kwargs)


def send_tokens(skale, sender_wallet, receiver_account, amount,
                wait_for=True):
    logger.info(
        f'Sending {amount} SKALE tokens from {sender_wallet.address} => '
        f'{receiver_account}'
    )

    wei_amount = skale.web3.toWei(amount, 'ether')
    tx_res = skale.token.transfer(receiver_account, wei_amount, wait_for=wait_for)
    if wait_for:
        check_receipt(tx_res.receipt)
    return tx_res


def send_ether(web3, sender_wallet, receiver_account, amount,
               gas_price=None, wait_for=True):
    logger.info(
        f'Sending {amount} ETH from {sender_wallet.address} => '
        f'{receiver_account}'
    )

    wei_amount = web3.toWei(amount, 'ether')
    tx = send_eth(web3, receiver_account, wei_amount, sender_wallet,
                  gas_price=gas_price)
    if wait_for:
        receipt = wait_for_receipt_by_blocks(web3, tx)
        check_receipt(receipt)
        return receipt
    else:  # pragma: no cover
        return tx


def account_eth_balance_wei(web3, address):
    return web3.eth.getBalance(address)


def check_ether_balance(web3, address):
    balance_wei = account_eth_balance_wei(web3, address)
    balance = web3.fromWei(balance_wei, 'ether')

    logger.info(f'{address} balance: {balance} ETH')
    return balance


def check_skale_balance(skale, address):
    balance_wei = skale.token.get_balance(address)
    balance = skale.web3.fromWei(balance_wei, 'ether')
    logger.info(f'{address} balance: {balance} SKALE')
    return balance


def generate_account(web3):
    account = web3.eth.account.create()
    private_key = account.key.hex()
    logger.info(f'Generated account: {account.address}')
    return {'address': account.address, 'private_key': private_key}


def generate_accounts(skale,
                      base_wallet,
                      n_wallets,
                      skale_amount,
                      eth_amount,
                      debug=False):
    n_wallets = int(n_wallets)
    results = []

    for _ in range(0, n_wallets):
        wallet = generate_account(skale.web3)

        send_tokens(skale, skale.wallet, wallet['address'], skale_amount)
        send_ether(skale.web3, skale.wallet, wallet['address'], eth_amount)

        if debug:
            check_ether_balance(skale.web3, wallet['address'])
            check_skale_balance(skale, wallet['address'])

        results.append(wallet)
        logger.info(LONG_LINE)

    return results



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.
""" SKALE base contract class """

import logging
from functools import wraps

from web3 import Web3

import skale.config as config
from skale.transactions.result import (TxRes, check_balance_and_gas,
                                       is_success, is_success_or_not_performed)
from skale.transactions.tools import make_dry_run_call, post_transaction
from skale.utils.account_tools import account_eth_balance_wei
from skale.utils.web3_utils import (wait_for_confirmation_blocks,
                                    wait_for_receipt_by_blocks)

from skale.utils.helper import to_camel_case


logger = logging.getLogger(__name__)


def execute_dry_run(skale, method, custom_gas_limit) -> tuple:
    dry_run_result = make_dry_run_call(skale, method, custom_gas_limit)
    estimated_gas_limit = None
    if is_success(dry_run_result):
        estimated_gas_limit = dry_run_result['payload']
    return dry_run_result, estimated_gas_limit


def transaction_method(transaction):
    @wraps(transaction)
    def wrapper(self, *args, wait_for=True,
                wait_timeout=4, blocks_to_wait=50, gas_limit=None,
                gas_price=None, nonce=None,
                dry_run_only=False, skip_dry_run=False,
                raise_for_status=True, confirmation_blocks=0, **kwargs):
        method = transaction(self, *args, **kwargs)
        dry_run_result, tx_hash, receipt = None, None, None

        # Make dry_run and estimate gas limit
        estimated_gas_limit = None
        if not skip_dry_run and not config.DISABLE_DRY_RUN:
            dry_run_result, estimated_gas_limit = execute_dry_run(
                self.skale, method, gas_limit
            )

        gas_limit = gas_limit or estimated_gas_limit or \
            config.DEFAULT_GAS_LIMIT

        # Check balance
        balance = account_eth_balance_wei(self.skale.web3,
                                          self.skale.wallet.address)
        gas_price = gas_price or config.DEFAULT_GAS_PRICE_WEI or \
            self.skale.gas_price
        balance_check_result = check_balance_and_gas(balance,
                                                     gas_price, gas_limit)
        rich_enough = is_success(balance_check_result)

        # Send transaction
        should_send_transaction = not dry_run_only and \
            is_success_or_not_performed(dry_run_result)

        if rich_enough and should_send_transaction:
            tx_hash = post_transaction(
                self.skale.wallet, method, gas_limit,
                gas_price, nonce
            )
            if wait_for:
                receipt = wait_for_receipt_by_blocks(
                    self.skale.web3,
                    tx_hash,
                    timeout=wait_timeout,
                    blocks_to_wait=blocks_to_wait
                )
            if confirmation_blocks:
                wait_for_confirmation_blocks(self.skale.web3,
                                             confirmation_blocks)

        tx_res = TxRes(dry_run_result, balance_check_result, tx_hash, receipt)

        if raise_for_status:
            tx_res.raise_for_status()
        return tx_res

    return wrapper


class BaseContract:
    def __init__(self, skale, name, address, abi):
        self.skale = skale
        self.name = name
        self.address = Web3.toChecksumAddress(address)
        self.contract = skale.web3.eth.contract(address=self.address, abi=abi)

    def __getattr__(self, attr):
        """Fallback for contract calls"""
        logger.debug("Calling contract function: "+attr)

        def wrapper(*args, **kw):
            logger.debug('called with %r and %r' % (args, kw))
            camel_case_fn_name = to_camel_case(attr)
            if hasattr(self.contract.functions, camel_case_fn_name):
                return getattr(self.contract.functions, camel_case_fn_name)(*args, **kw).call()
            if hasattr(self.contract.functions, attr):
                return getattr(self.contract.functions, attr)(*args, **kw).call()
            raise AttributeError(attr)
        return wrapper



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

from skale.contracts.base_contract import BaseContract, transaction_method


class TokenLaunchManager(BaseContract):
    """Wrapper for TokenLaunchManager.sol functions"""

    @transaction_method
    def approve_batch_of_transfers(self, wallet_addresses, values):
        if len(wallet_addresses) != len(values):
            raise ValueError('wallet_addresses and values length do not match')
        return self.contract.functions.approveBatchOfTransfers(wallet_addresses, values)

    @transaction_method
    def approve_transfer(self, wallet_address, value):
        return self.contract.functions.approveBatchOfTransfers(wallet_address, value)

    @transaction_method
    def retrieve(self):
        return self.contract.functions.retrieve()

    @transaction_method
    def complete_token_launch(self):
        return self.contract.functions.completeTokenLaunch()

    def approved(self, address: str) -> int:
        return self.contract.functions.approved(address).call()



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.
""" SKALE manager operations """


import logging
import socket

from eth_abi import encode_abi

from skale.contracts.base_contract import BaseContract, transaction_method
from skale.utils import helper
from skale.transactions.result import TxRes

logger = logging.getLogger(__name__)


class Manager(BaseContract):

    @transaction_method
    def create_node(self, ip, port, name, domain_name, public_ip=None):
        logger.info(
            f'create_node: {ip}:{port}, name: {name}, domain_name: {domain_name}')
        skale_nonce = helper.generate_nonce()
        if not public_ip:
            public_ip = ip
        ip_bytes = socket.inet_aton(ip)
        public_ip_bytes = socket.inet_aton(public_ip)
        pk_parts_bytes = helper.split_public_key(self.skale.wallet.public_key)
        return self.contract.functions.createNode(
            port,
            skale_nonce,
            ip_bytes,
            public_ip_bytes,
            pk_parts_bytes,
            name,
            domain_name
        )

    def create_default_schain(self, name):
        lifetime = 3600
        nodes_type = 4
        price_in_wei = self.skale.schains.get_schain_price(
            nodes_type, lifetime)
        return self.create_schain(lifetime, nodes_type, price_in_wei, name,
                                  wait_for=True)

    @transaction_method
    def create_schain(self, lifetime, type_of_nodes, deposit, name):
        logger.info(
            f'create_schain: type_of_nodes: {type_of_nodes}, name: {name}')
        skale_nonce = helper.generate_nonce()
        tx_data = encode_abi(
            ['uint', 'uint8', 'uint16', 'string'],
            [lifetime, type_of_nodes, skale_nonce, name]
        )
        return self.skale.token.contract.functions.send(self.address, deposit, tx_data)

    @transaction_method
    def get_bounty(self, node_id):
        return self.contract.functions.getBounty(node_id)

    @transaction_method
    def delete_schain(self, schain_name):
        return self.contract.functions.deleteSchain(schain_name)

    @transaction_method
    def node_exit(self, node_id):
        return self.contract.functions.nodeExit(node_id)

    @transaction_method
    def grant_role(self, role: bytes, address: str) -> TxRes:
        return self.contract.functions.grantRole(role, address)

    def default_admin_role(self) -> bytes:
        return self.contract.functions.DEFAULT_ADMIN_ROLE().call()

    def admin_role(self) -> bytes:
        return self.contract.functions.ADMIN_ROLE().call()

    def has_role(self, role: bytes, address: str) -> bool:
        return self.contract.functions.hasRole(role, address).call()



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

NODE_DEPOSIT = 100000000000000000000
GAS_LIMIT_COEFFICIENT = 1.2
GAS_PRICE_COEFFICIENT = 2

LONG_LINE = '=' * 100


SCHAIN_TYPES = {
    'tiny': 1,
    'small': 2,
    'medium': 3,
    'test': 4,
}



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

from skale.contracts.base_contract import BaseContract, transaction_method
from skale.transactions.tools import retry_tx
from skale.utils.helper import split_public_key


class KeyShare:
    def __init__(self, public_key: str, share: bytes):
        self.public_key = split_public_key(public_key)
        self.share = share
        self.tuple = (self.public_key, self.share)


class G2Point:
    def __init__(self, xa, xb, ya, yb):
        self.x = (xa, xb)
        self.y = (ya, yb)
        self.tuple = (self.x, self.y)


class DKG(BaseContract):
    @retry_tx
    @transaction_method
    def broadcast(self, group_index, node_index,
                  verification_vector, secret_key_contribution):
        return self.contract.functions.broadcast(group_index, node_index,
                                                 verification_vector,
                                                 secret_key_contribution)

    @retry_tx
    @transaction_method
    def pre_response(
        self,
        group_index: str,
        from_node_index: int,
        verification_vector: list,
        verification_vector_mult: list,
        secret_key_contribution: list
    ):
        return self.contract.functions.preResponse(
            schainId=group_index,
            fromNodeIndex=from_node_index,
            verificationVector=verification_vector,
            verificationVectorMult=verification_vector_mult,
            secretKeyContribution=secret_key_contribution,
        )

    @retry_tx
    @transaction_method
    def response(
            self,
            group_index: bytes,
            from_node_index: int,
            secret_number: int,
            multiplied_share: G2Point
    ):
        return self.contract.functions.response(
            schainId=group_index,
            fromNodeIndex=from_node_index,
            secretNumber=secret_number,
            multipliedShare=multiplied_share
        )

    @retry_tx
    @transaction_method
    def alright(self, group_index, from_node_index):
        return self.contract.functions.alright(group_index, from_node_index)

    @retry_tx
    @transaction_method
    def complaint(self, group_index, from_node_index, to_node_index):
        return self.contract.functions.complaint(group_index, from_node_index,
                                                 to_node_index)

    @retry_tx
    @transaction_method
    def complaint_bad_data(self, group_index, from_node_index, to_node_index):
        return self.contract.functions.complaintBadData(group_index,
                                                        from_node_index,
                                                        to_node_index)

    def is_last_dkg_successful(self, group_index):
        return self.contract.functions.isLastDKGSuccessful(group_index).call()

    def is_channel_opened(self, group_index):
        return self.contract.functions.isChannelOpened(group_index).call()

    def is_broadcast_possible(self, group_index, node_id, address):
        return self.contract.functions.isBroadcastPossible(group_index, node_id).call(
            {'from': address}
        )

    def is_alright_possible(self, group_index, node_id, address):
        return self.contract.functions.isAlrightPossible(group_index, node_id).call(
            {'from': address}
        )

    def is_complaint_possible(self, group_index, node_from, node_to, address):
        return self.contract.functions.isComplaintPossible(group_index, node_from, node_to).call(
            {'from': address}
        )

    def is_pre_response_possible(self, group_index, node_id, address):
        return self.contract.functions.isPreResponsePossible(group_index, node_id).call(
            {'from': address}
        )

    def is_response_possible(self, group_index, node_id, address):
        return self.contract.functions.isResponsePossible(group_index, node_id).call(
            {'from': address}
        )

    def is_all_data_received(self, group_index, node_from):
        return self.contract.functions.isAllDataReceived(group_index, node_from).call()

    def is_everyone_broadcasted(self, group_index, address):
        return self.contract.functions.isEveryoneBroadcasted(group_index).call(
            {'from': address}
        )

    def get_number_of_completed(self, group_index):
        return self.contract.functions.getNumberOfCompleted(group_index).call()

    def get_channel_started_time(self, group_index):
        return self.contract.functions.getChannelStartedTime(group_index).call()

    def get_complaint_started_time(self, group_index):
        return self.contract.functions.getComplaintStartedTime(group_index).call()

    def get_alright_started_time(self, group_index):
        return self.contract.functions.getAlrightStartedTime(group_index).call()

    def get_complaint_data(self, group_index):
        return self.contract.functions.getComplaintData(group_index).call()

    def get_time_of_last_successful_dkg(self, group_index):
        return self.contract.functions.getTimeOfLastSuccesfulDKG(group_index).call()



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.
""" SKALE web3 utilities """

import logging
import time
from urllib.parse import urlparse

from eth_keys import keys
from web3 import Web3, WebsocketProvider, HTTPProvider
from web3.exceptions import TransactionNotFound

logger = logging.getLogger(__name__)


WS_MAX_MESSAGE_DATA_BYTES = 5 * 1024 * 1024
MAX_WAITING_TIME = 3 * 60 * 60  # 3 hours


def get_provider(endpoint, timeout=10, request_kwargs={}):
    scheme = urlparse(endpoint).scheme
    if scheme == 'ws' or scheme == 'wss':
        kwargs = request_kwargs or {'max_size': WS_MAX_MESSAGE_DATA_BYTES}
        return WebsocketProvider(endpoint, websocket_timeout=timeout,
                                 websocket_kwargs=kwargs)

    if scheme == 'http' or scheme == 'https':
        kwargs = {'timeout': timeout, **request_kwargs}
        return HTTPProvider(endpoint, request_kwargs=kwargs)

    raise Exception(
        'Wrong endpoint option.'
        'Supported endpoint schemes: http/https/ws/wss'
    )


def init_web3(endpoint):
    provider = get_provider(endpoint)
    return Web3(provider)


def get_receipt(web3, tx):
    return web3.eth.getTransactionReceipt(tx)


def get_eth_nonce(web3, address):
    return web3.eth.getTransactionCount(address)


def wait_for_receipt_by_blocks(web3, tx, timeout=4, blocks_to_wait=50):
    previous_block = web3.eth.blockNumber
    current_block = previous_block
    wait_start_time = time.time()
    while time.time() - wait_start_time < MAX_WAITING_TIME and \
            current_block <= previous_block + blocks_to_wait:
        try:
            receipt = get_receipt(web3, tx)
        except TransactionNotFound:
            receipt = None
        if receipt is not None:
            return receipt
        current_block = web3.eth.blockNumber
        time.sleep(timeout)
    raise TransactionNotFound(f"Transaction with hash: {tx} not found.")


def wait_receipt(web3, tx, retries=30, timeout=5):
    for _ in range(0, retries):
        try:
            receipt = get_receipt(web3, tx)
        except TransactionNotFound:
            receipt = None
        if receipt is not None:
            return receipt
        time.sleep(timeout)  # pragma: no cover
    raise TransactionNotFound(f"Transaction with hash: {tx} not found.")


def check_receipt(receipt, raise_error=True):
    if receipt['status'] != 1:  # pragma: no cover
        if raise_error:
            raise ValueError("Transaction failed, see receipt", receipt)
        else:
            return False
    return True


def wait_for_confirmation_blocks(web3, blocks_to_wait, request_timeout=5):
    current_block = start_block = web3.eth.blockNumber
    logger.info(
        f'Current block number is {current_block}, '
        f'waiting for {blocks_to_wait} confimration blocks to be mined'
    )
    wait_start_time = time.time()
    while time.time() - wait_start_time < MAX_WAITING_TIME and \
            current_block <= start_block + blocks_to_wait:
        current_block = web3.eth.blockNumber
        time.sleep(request_timeout)


def private_key_to_public(pr):
    pr_bytes = Web3.toBytes(hexstr=pr)
    pk = keys.PrivateKey(pr_bytes)
    return pk.public_key


def public_key_to_address(pk):
    hash = Web3.keccak(hexstr=str(pk))
    return Web3.toHex(hash[-20:])


def private_key_to_address(pr):
    pk = private_key_to_public(pr)
    return public_key_to_address(pk)


def to_checksum_address(address):
    return Web3.toChecksumAddress(address)


def wallet_to_public_key(wallet):
    if isinstance(wallet, dict):
        return private_key_to_public(wallet['private_key'])
    else:
        return wallet['public_key']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import (
    find_packages,
    setup,
)

extras_require = {
    'linter': [
        "flake8==3.8.3",
        "isort>=4.2.15,<5.4.3",
    ],
    'dev': [
        "bumpversion==0.6.0",
        "pytest==5.4.3",
        "click==7.1.2",
        "twine==3.1.1",
        "mock==4.0.2",
        "when-changed",
        "Random-Word==1.0.4",
        "pytest-cov==2.8.1"
    ],
    'hw-wallet': [
        "ledgerblue==0.1.31"
    ]
}

extras_require['dev'] = (
    extras_require['linter'] + extras_require['dev'] + extras_require['hw-wallet']
)

setup(
    name='skale.py',
    version='5.0dev10',
    description='SKALE client tools',
    long_description_markdown_filename='README.md',
    author='SKALE Labs',
    author_email='support@skalelabs.com',
    url='https://github.com/skalenetwork/skale.py',
    include_package_data=True,
    install_requires=[
        "web3==5.13.1",
        "asyncio==3.4.3",
        "pyyaml==5.3.1",
        "sgx.py==0.7dev2"
    ],

    python_requires='>=3.7,<4',
    extras_require=extras_require,

    keywords='skale',
    packages=find_packages(exclude=['tests']),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
    ]
)



#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

from skale.utils.exceptions import SChainNotFoundException
from skale.schain_config import PORTS_PER_SCHAIN


def calc_schain_base_port(node_base_port, schain_index):
    return node_base_port + schain_index * PORTS_PER_SCHAIN


def get_schain_index_in_node(schain_name, node_schains):
    for index, schain in enumerate(node_schains):
        if schain_name == schain['name']:
            return index
    raise SChainNotFoundException(f'sChain {schain_name} is not found in the list: {node_schains}')


def get_schain_base_port_on_node(schains_on_node, schain_name, node_base_port):
    schain_index = get_schain_index_in_node(schain_name, schains_on_node)
    return calc_schain_base_port(node_base_port, schain_index)
